import asyncio

import aiocron
import grpc
import logging

from concurrent import futures
from perxis.collections import collections_pb2_grpc
from perxis.environments import environments_pb2_grpc
from perxis.roles import roles_pb2_grpc
from perxis.clients import clients_pb2_grpc
from perxis.extensions import extension_pb2_grpc, manager_pb2_grpc, manager_pb2
from perxis.interceptors import header_adder_interceptor

logger = logging.getLogger(__name__)


async def _main(
        ext_descriptor: manager_pb2.ExtensionDescriptor,
        servicer_cls: extension_pb2_grpc.ExtensionServicer,
        ext_manager_host: str,
        content_host: str
):
    interceptor = header_adder_interceptor(
        'x-perxis-access', 'system'
    )

    @aiocron.crontab('* * * * *', start=False)
    async def register_extension():
        await ext_manager_stub.RegisterExtensions(manager_pb2.RegisterExtensionsRequest(
            extensions=[ext_descriptor]
        ))
        logger.info(f"Расширение {ext_descriptor.extension} зарегистрировано в perxis")

    async with grpc.aio.insecure_channel(ext_manager_host, interceptors=[interceptor]) as extensions_manager_channel:
        ext_manager_stub = manager_pb2_grpc.ExtensionManagerStub(extensions_manager_channel)

        registered_extensions: manager_pb2.ListExtensionsResponse = await ext_manager_stub.ListExtensions(
            manager_pb2.ListExtensionsRequest()
        )

        # todo enable after fix - https://tracker.yandex.ru/PRXS-1507
        # for ext in registered_extensions.extensions:
        #     if ext.extension == ext_descriptor.extension:
        #         if ext.version != ext_descriptor.version:
        #             ext_manager_stub.UnregisterExtensions(
        #                 manager_pb2.UnregisterExtensionsRequest(
        #                     extensions=[ext_descriptor]
        #                 )
        #             )
        #
        #             logger.info(
        #                 f"Изменилась версия расширения {ext_descriptor.extension} ({ext.version} -> {ext_descriptor.version})"
        #             )

        register_extension.start()

        async with grpc.aio.insecure_channel(content_host, interceptors=[interceptor]) as content_channel:

            collections_stub = collections_pb2_grpc.CollectionsStub(content_channel)
            roles_stub = roles_pb2_grpc.RolesStub(content_channel)
            clients_stub = clients_pb2_grpc.ClientsStub(content_channel)
            environments_stub = environments_pb2_grpc.EnvironmentsStub(content_channel)

            server = grpc.aio.server(futures.ThreadPoolExecutor(max_workers=10))
            extension_pb2_grpc.add_ExtensionServicer_to_server(
                servicer_cls(
                    collections_stub, environments_stub, roles_stub, clients_stub
                ), server
            )
            server.add_insecure_port("[::]:50051")
            await server.start()
            await server.wait_for_termination()


def bootstrap(
        ext_descriptor: manager_pb2.ExtensionDescriptor,
        servicer_cls: extension_pb2_grpc.ExtensionServicer,
        ext_manager_host: str,
        content_host: str,
):

    logger.info(f"Инициализация сервиса расширения {ext_descriptor.extension}")

    loop = asyncio.get_event_loop()

    try:
        loop.run_until_complete(
            _main(
                ext_descriptor,
                servicer_cls,
                ext_manager_host,
                content_host
            )
        )
    finally:
        loop.close()
        loop.stop()
        logging.info('Successfully shutdown service')
